// database field container                                      -*- C++ -*-
// $Id: field.h,v 1.1 2004/01/28 21:21:07 roger Exp $
//
// Copyright (C) 2003  Roger Leigh <rleigh@debian.org>
//
//
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//  * Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above
//    copyright notice, this list of conditions and the following
//    disclaimer in the documentation and/or other materials provided
//    with the distribution.
//  * Neither the name of the author, nor the names of other
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
//  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
//  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
//  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
//  SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////

#ifndef PQXX_OBJECT_FIELD_H
#define PQXX_OBJECT_FIELD_H

#include <sigc++/signal.h>

namespace pqxxobject
{
  /**
   * Database field template class.
   * This class is used to represent a single field in a row of a
   * table.  This is a single value belonging to a column in a row,
   * rather than the whole column.
   *
   * As well as storing value, the class has the ability to emit
   * signals when the field value is changed.  Listeners (e.g. user
   * interface widgets) may connect to the signal and will receive
   * notification of changes as they occur.
   */
  template<typename T>
  class field
  {
  public:
    typedef T value_type;

    /// The constructor.
    field():
      m_value()
    {}

    /// The constructor.
    field(const value_type& value):
      m_value(value)
    {}

    /// The copy constructor.
    field(const field<value_type>& rhs):
      m_value(rhs.m_value),
      m_signal_changed()
    {}

    virtual ~field()
    {}

    /// Access member functions.
    const value_type *operator -> () const
    {
      return &m_value;
    }

    const value_type& operator * () const
    {
      return m_value;
    }

    /// Overloaded assignment operator.
    field<value_type>& operator = (const field<value_type>& rhs)
    {
      set_value(rhs.m_value);
      return *this;
    }

    /// Overloaded assignment operator.
    field<value_type>& operator = (const value_type& rhs)
    {
      set_value(rhs);
      return *this;
    }

    /// Conversion operator.
    operator const value_type& () const
    {
      return m_value;
    }

    /**
     * Get the contained value by reference.
     * @returns a reference to the value.
     */
    value_type& get_value()
    {
      return m_value;
    }

    /**
     * Get the contained value by constant reference.
     * @returns a constant reference to the value.
     */
    const value_type& get_value() const
    {
      return m_value;
    }

    /**
     * Set the contained value.
     * @param value the value to set.
     */
    void set_value(const value_type& value)
    {
      m_value = value;
      m_signal_changed.emit();
    }

    /**
     * Signal emitted on value change.
     * @returns the signal.
     *
     * For example:
     * @code
     * field<int> col;
     * someclass listener;
     * col.signal_changed().connect
     *   ( SigC::slot(listener, &someclass::on_col_changed() );
     * @endcode
     * i.e. a class method (listener.on_col_changed()) will be called
     * when the value is changed.
     */
    SigC::Signal0<void>& signal_changed()
    {
      return m_signal_changed;
    }

  private:
    /// The contained value.
    value_type m_value;
    /// The changed signal.
    SigC::Signal0<void> m_signal_changed;

  }; // class field

}; // namespace pqxxobject

#endif // PQXX_OBJECT_FIELD_H
