// database row template class                                       -*- C++ -*-
// $Id: row.h,v 1.7 2004/01/28 21:21:08 roger Exp $
//
// Copyright (C) 2003  Roger Leigh <rleigh@debian.org>
//
//
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//  * Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above
//    copyright notice, this list of conditions and the following
//    disclaimer in the documentation and/or other materials provided
//    with the distribution.
//  * Neither the name of the author, nor the names of other
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
//  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
//  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
//  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
//  SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////

#ifndef PQXX_OBJECT_ROW_H
#define PQXX_OBJECT_ROW_H

#include <memory>

#include <pqxx-object/row_base.h>

namespace pqxxobject
{
  /**
   * Database row template class.
   * The class represents a single row in a database table (or result set).
   * This class should be derived from by your own row class.  For example:
   * @code
   * class Address : public libpqxxobject::row<Address>
   * {
   * private:
   *   libpqxx::column<int> m_id; // Primary key
   *   libpqxx::column<std::string> name; // Name
   *   libpqxx::column<std::string> street; // Street
   * [...]
   * }
   * @endcode
   * Your class should provide:
   *   - Member variables of type libpqxx::column<> for each column in the row.
   *   - Getter and setter methods (where appropriate) for each column.
   *   - Signal methods (where appropriate) for each column.
   *   - m_modified should be set to true when a column is changed.
   *   - A create() method that returns a row_ptr.  This has the following prototype:
   *     @code
   * static row_ptr create(pqxx::result::const_iterator row,
   *                       pqxxobject::transaction& tran);
   *     @endcode
   *     This will be used by the companion table object (if any) when
   *     recreating objects from the database.
   *   - Implementations of insert_impl(), update_impl(), erase_impl()
   *     and refresh_impl().  These are defined in row_base.  If any
   *     of these are not required, simply ignore them, and the default
   *     (null) methods will be used instead.
   */
  template<typename Row>
  class row : public row_base
  {
  public:
    /// Row object pointer.
    typedef std::auto_ptr<Row> row_ptr;

    /// The constructor.
    row():
      row_base()
    {}

    /**
     * The constructor.
     * @param status the row status.
     * @param modified true if the row is modified, otherwise false.
     */
    row(row_state status,
	bool modified=false):
      row_base(status, modified)
    {}

    /// The destructor.
    virtual ~row()
    {}

    /**
     * Create a new row object from an SQL result set.
     * @param row an interator pointing to a row of an SQL result set.
     * @param tran a PostgreSQL transaction object.
     * @returns a Place.
     */
    static row_ptr create(pqxx::result::const_iterator row,
			  pqxxobject::transaction& tran)
    {
      row_ptr p(new Row);
      p->convert_impl(row, tran);
      p->row_base::m_state = STATE_INITIALISED;
      return p;
    }

  protected:

  }; // class row

}; // namespace pqxxobject

#endif // PQXX_OBJECT_ROW_H
