// database row base class                                       -*- C++ -*-
// $Id: row_base.cc,v 1.5 2004/01/28 21:21:08 roger Exp $
//
// Copyright (C) 2003  Roger Leigh <rleigh@debian.org>
//
//
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//  * Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above
//    copyright notice, this list of conditions and the following
//    disclaimer in the documentation and/or other materials provided
//    with the distribution.
//  * Neither the name of the author, nor the names of other
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
//  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
//  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
//  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
//  SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////

#include <pqxx-object/row_base.h>

using namespace pqxxobject;

namespace
{
  std::string describe_status(row_base::row_state current_state)
  {
    std::string status;
    switch (current_state)
      {
      case row_base::STATE_UNINITIALISED:
	status = "the row is not initialised";
	break;
      case row_base::STATE_INITIALISED:
	status = "the row is initialised";
	break;
      case row_base::STATE_INCONSISTENT:
	status = "the row is in an inconsistent state";
	break;
      default:
	status = "the row is in an unknown state";
      }
    return status;
  }
}

row_base::row_base():
  m_state(STATE_UNINITIALISED),
  m_modified(false)
{
}

row_base::row_base(row_state status,
		   bool modified):
  m_state(status),
  m_modified(modified)
{
}

row_base::row_base(const row_base& rhs):
  SigC::Object(),
  m_state(rhs.m_state),
  m_modified(rhs.m_modified)
{
}

row_base::~row_base()
{
}

row_base&
row_base::operator = (const row_base& rhs)
{
  m_state = rhs.m_state;
  m_modified = rhs.m_modified;
  return *this;
}

row_base::row_state
row_base::get_state() const
{
  return m_state;
}

bool
row_base::is_modified() const
{
  return m_modified;
}

void
row_base::refresh(pqxxobject::transaction& tran)
{
  if (m_state != STATE_UNINITIALISED)
    {
      m_state = STATE_INCONSISTENT;
      refresh_impl(tran);
      m_signal_refreshed.emit();
      m_signal_changed.emit();
    }
  else
    throw DatabaseError("The row could not be refreshed: " + describe_status(m_state));
}

void
row_base::insert(pqxxobject::transaction& tran)
{
  if (m_state == STATE_UNINITIALISED)
    {
      // Register for refresh on transaction commit/abort.
      tran.signal_commit().connect
	(SigC::slot(*this, &row_base::refresh) );
      tran.signal_abort().connect
	(SigC::slot(*this, &row_base::refresh) );
      m_state = STATE_INCONSISTENT;
      insert_impl(tran);
      m_modified = false;
      m_signal_inserted.emit();
    }
  else
    throw DatabaseError("The row could not be inserted: " + describe_status(m_state));
}

void
row_base::update(pqxxobject::transaction& tran)
{
  if (m_state == STATE_INITIALISED)
    {
      // Register for refresh on transaction commit/abort.
      tran.signal_commit().connect
	( SigC::slot(*this, &row_base::refresh) );
      tran.signal_abort().connect
	( SigC::slot(*this, &row_base::refresh) );
      m_state = STATE_INCONSISTENT;
      update_impl(tran);
      m_modified = false;
      m_signal_updated.emit();
    }
  else
    throw DatabaseError("The row could not be updated: " + describe_status(m_state));
}

void
row_base::erase(pqxxobject::transaction& tran)
{
  if (m_state == STATE_INITIALISED)
    {
      m_state = STATE_INCONSISTENT;
      erase_impl(tran);
      m_signal_erased.emit();
      m_signal_changed.emit();
    }
  else
    throw DatabaseError("The row could not be deleted: " + describe_status(m_state));
}

SigC::Signal0<void>&
row_base::signal_changed()
{
  return m_signal_changed;
}

SigC::Signal0<void>&
row_base::signal_refreshed()
{
  return m_signal_refreshed;
}

SigC::Signal0<void>&
row_base::signal_inserted()
{
  return m_signal_inserted;
}

SigC::Signal0<void>&
row_base::signal_updated()
{
  return m_signal_updated;
}

SigC::Signal0<void>&
row_base::signal_erased()
{
  return m_signal_erased;
}

void
row_base::refresh_impl(pqxxobject::transaction& tran)
{
}

void
row_base::insert_impl(pqxxobject::transaction& tran)
{
}

void
row_base::update_impl(pqxxobject::transaction& tran)
{
}

void
row_base::erase_impl(pqxxobject::transaction& tran)
{
}
