// database row base class                                       -*- C++ -*-
// $Id: row_base.h,v 1.5 2004/01/28 21:21:08 roger Exp $
//
// Copyright (C) 2003  Roger Leigh <rleigh@debian.org>
//
//
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//  * Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above
//    copyright notice, this list of conditions and the following
//    disclaimer in the documentation and/or other materials provided
//    with the distribution.
//  * Neither the name of the author, nor the names of other
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
//  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
//  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
//  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
//  SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////

#ifndef PQXX_OBJECT_ROW_BASE_H
#define PQXX_OBJECT_ROW_BASE_H

#include <sigc++/object.h>
#include <sigc++/signal.h>
#include <sigc++/object_slot.h>

#include <pqxx-object/transaction.h>

namespace pqxxobject
{
  /**
   * Database row base class.
   * This class provides the functionality common to all database rows
   * of all types.  Because it derives from SigC::Object, signals may
   * be connected to it, and all classes which derive from it.
   */
  class row_base : public SigC::Object
  {
  public:
    /// The state of the row
    enum row_state
      {
	/**
	 *The row is uninitialised (either new or deleted), not
	 *existing in the database.
	 */
	STATE_UNINITIALISED,
	/// The row exists in the database.
	STATE_INITIALISED,
	/**
	 * The row exists in the database, but is inconsistent due to
	 * an unsuccessful or partially competed transaction.
	 */
	STATE_INCONSISTENT
      };

    /// The constructor.
    row_base();

    /**
     * The constructor.
     * @param status the row status.
     * @param modified true if the row is modified, otherwise false.
     */
    row_base(row_state status,
	     bool modified=false);

    /** The copy constructor.
     * @param rhs the object to copy.
     */
    row_base(const row_base& rhs);

    /// The destructor.
    virtual ~row_base();


    /**
     * Overloaded assignment operator.
     * @param rhs the object to assign.
     */
    row_base& operator = (const row_base& rhs);

    /**
     * Get the row status.
     * @returns the row status.
     */
    row_state get_state() const;

    /**
     * Get the row modification status.
     * @returns true if the row has been modified, false otherwise.
     */
    bool is_modified() const;

    /**
     * Refresh a row from a table.
     * @param tran the transaction which will be used to execute queries.
     */
    void refresh(pqxxobject::transaction& tran);

    /**
     * Insert a row into a table.
     * @param tran the transaction which will be used to execute queries.
     */
    void insert(pqxxobject::transaction& tran);

    /**
     * Update a row from a table.
     * @param tran the transaction which will be used to execute queries.
     */
    void update(pqxxobject::transaction& tran);

    /**
     * Remove a row from a table.
     * @param tran the transaction which will be used to execute queries.
     */
    void erase(pqxxobject::transaction& tran);

    /**
     * Signal emitted on row change.
     * @returns the signal.
     */
    SigC::Signal0<void>& signal_changed();

    /**
     * Signal emitted on refresh.
     * @returns the signal.
     */
    SigC::Signal0<void>& signal_refreshed();

    /**
     * Signal emitted on insertion.
     * @returns the signal.
     */
    SigC::Signal0<void>& signal_inserted();

    /**
     * Signal emitted on update.
     * @returns the signal.
     */
    SigC::Signal0<void>& signal_updated();

    /**
     * Signal emitted on deletion.
     * @returns the signal.
     */
    SigC::Signal0<void>& signal_erased();

  protected:
    /**
     * Refresh a row from a table.
     * This default method does nothing and should be overridden by
     * derived classes.
     * @param tran the transaction which will be used to execute queries.
     */
    virtual void refresh_impl(pqxxobject::transaction& tran);

    /**
     * Insert a row into a table.
     * This default method does nothing and should be overridden by
     * derived classes.
     * @param tran the transaction which will be used to execute queries.
     */
    virtual void insert_impl(pqxxobject::transaction& tran);

    /**
     * Update a row from a table.
     * This default method does nothing and should be overridden by
     * derived classes.
     * @param tran the transaction which will be used to execute queries.
     */
    virtual void update_impl(pqxxobject::transaction& tran);

    /**
     * Remove a row from a table.
     * This default method does nothing and should be overridden by
     * derived classes.
     * @param tran the transaction which will be used to execute queries.
     */
    virtual void erase_impl(pqxxobject::transaction& tran);

    /// The row state.
    row_state m_state;
    /// The row modification state.
    bool m_modified;

    /// The changed signal.
    SigC::Signal0<void> m_signal_changed;
    /// The refreshed signal.
    SigC::Signal0<void> m_signal_refreshed;
    /// The inserted signal.
    SigC::Signal0<void> m_signal_inserted;
    /// The updated signal.
    SigC::Signal0<void> m_signal_updated;
    /// The erased signal.
    SigC::Signal0<void> m_signal_erased;

  }; // class row_base

}; // namespace pqxxobject

#endif // PQXX_OBJECT_ROW_BASE_H
