// database table convenience wrapper                            -*- C++ -*-
// $Id: table.h,v 1.7 2004/01/28 21:21:08 roger Exp $
//
// Copyright (C) 2003  Roger Leigh <rleigh@debian.org>
//
//
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//  * Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above
//    copyright notice, this list of conditions and the following
//    disclaimer in the documentation and/or other materials provided
//    with the distribution.
//  * Neither the name of the author, nor the names of other
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
//  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
//  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
//  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
//  SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////

#ifndef PQXX_OBJECT_TABLE_H
#define PQXX_OBJECT_TABLE_H

#include <list>
#include <memory>

#include <pqxx-object/row.h>

namespace pqxxobject
{
  /**
   * Database table convenience wrapper.
   * This class provides methods to simplify table access for classes
   * which represent database tables.  Methods are provided to find
   * single and multiple rows and to insert, update and delete rows
   * from the table.
   */
  template<typename Row>
  class table
  {
  public:
    /// The type of the row the table contains.
    typedef Row row_type;
    /// Row object pointer.
    typedef typename Row::row_ptr row_ptr;
    /// List of rows.
    typedef std::list<Row> row_list;
    /// List of rows pointer.
    typedef std::auto_ptr<row_list> row_list_ptr;


  protected:
    /**
     * The constructor.
     * @param tran a pointer to a PostgreSQL transaction
     * object.  This must have been previously created from
     * connection.  If set to NULL, table will run in auto-commit
     * mode, and will create a new transaction for each database
     * operation.
     */
    table(pqxxobject::transaction& tran):
      m_transaction(tran)
    {}

  public:
    /// The destructor.
    virtual ~table()
    {}

    /**
     * Add a new row.
     * @param row the row to add.
     */
    virtual
    void
    insert(row_type& row)
    {
      row.insert(m_transaction);
    }

    /**
     * Update the details of an existing row.
     * @param row the row to update.
     */
    virtual
    void
    update(row_type& row)
    {
      row.update(m_transaction);
    }

    /**
     * Erase (remove) a row.
     * @param row the row to erase.
     */
    virtual
    void
    erase(row_type& row)
    {
      row.erase(m_transaction);
    }

    /**
     * Refresh a row (synchronise with current database state).
     * @param row the row to refresh.
     */
    virtual
    void
    refresh(row_type& row)
    {
      row.refresh(m_transaction);
    }

  protected:
    /**
     * Find a single row from the database.
     * @param query the SQL query to run.
     * @returns a pointer to row, or NULL if no member was found.
     */
    virtual
    row_ptr
    find_one(const std::string& query)
    {
      try
	{
	  m_transaction.begin("pqxxobject::table::find_one");

	  pqxx::result R (m_transaction.exec(query));


	  row_ptr ret;

	  if (R.size() > 0)
	    {
	      pqxx::result::const_iterator cur = R.begin();
	      // Assign result (if any) to temporary row.
	      if (cur != R.end())
		ret = row_type::create(cur, m_transaction);
	    }

	  m_transaction.commit();
	  return ret;
	}
      catch (const std::exception& e)
	{
	  m_transaction.abort();
	  throw DatabaseError(e.what());
	}
    }

    /**
     * Find multiple rows from the database.
     * @param query the SQL query to run.
     * @returns a pointer to a vector of rows.
     */
    virtual
    row_list_ptr
    find_many(const std::string& query)
    {
      try
	{
	  m_transaction.begin("pqxxobject::table::find_many");

	  pqxx::result R (m_transaction.exec(query));

	  row_list_ptr ret(new row_list);

	  for (pqxx::result::const_iterator cur = R.begin(); cur != R.end(); cur++)
	    {
	      row_ptr tmp_row;
	      // Assign results (one row) to temporary row
	      tmp_row = row_type::create(cur, m_transaction);
	      // Add this member to the list
	      ret->push_back(*tmp_row);
	    }

	  m_transaction.commit();

	  return ret;
	}
      catch (const std::exception& e)
	{
	  m_transaction.abort();
	  throw DatabaseError(e.what());
	}
    }

    pqxxobject::transaction& m_transaction;

  }; // class table

}; // namespace pqxxobject

#endif // PQXX_OBJECT_TABLE_H
