// database transaction convenience wrapper                      -*- C++ -*-
// $Id: transaction.cc,v 1.7 2004/01/28 21:21:08 roger Exp $
//
// Copyright (C) 2003  Roger Leigh <rleigh@debian.org>
//
//
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//  * Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above
//    copyright notice, this list of conditions and the following
//    disclaimer in the documentation and/or other materials provided
//    with the distribution.
//  * Neither the name of the author, nor the names of other
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
//  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
//  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
//  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
//  SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////

#include <cassert>
#include <iostream>

#include <sigc++/object.h>

#include <pqxx-object/transaction.h>

using namespace pqxxobject;

namespace
{
  std::string describe_status(transaction::state current_state)
  {
    std::string status;
    switch (current_state)
      {
      case transaction::STATE_EXECUTING:
	status = "the current transaction is executing normally";
	break;
      case transaction::STATE_ABORTING:
	status = "the current transaction is in the process of aborting";
	break;
      case transaction::STATE_COMMITTING:
	status = "the current transaction is in the process of committing";
	break;
      case transaction::STATE_NONE:
	status = "there is no current transaction";
	break;
      case transaction::STATE_ABORTED:
	status = "the current transaction is aborted";
	break;
      case transaction::STATE_COMMITTED:
	status = "the current transaction is committed";
	break;
      default:
	status = "the current transaction status is unknown";
      }
    return status;
  }
}

transaction::transaction(pqxx::connection& conn,
			 pqxx::transaction<>*& tran):
  m_connection(conn),
  m_transaction(tran),
  m_state(STATE_NONE),
  m_recursion_level(0)
{
}

transaction::~transaction()
{
}

SigC::Signal1<void, pqxxobject::transaction&>&
transaction::signal_commit()
{
  return m_signal_commit;
}

SigC::Signal1<void, pqxxobject::transaction&>&
transaction::signal_abort()
{
  return m_signal_abort;
}

void
transaction::begin(const std::string& name)
{
//   std::cerr << "transaction::begin";
  try
    {
      ++m_recursion_level;
//       std::cerr << "(" << m_recursion_level << ")" << std::endl;
      if (m_recursion_level == 1)
	_begin(name);
    }
  catch (const std::exception& e)
    {
      _end();
      throw DatabaseError(e.what());
    }
}

void
transaction::end()
{
//   std::cerr << "transaction::end";
//   std::cerr << "(" << m_recursion_level << ")" << std::endl;
  try
    {
      if (m_recursion_level > 0)
	{
	  --m_recursion_level;
	  if (m_recursion_level == 0)
	    {
	      _end();
	    }
	}
      else
	_end();
    }
  catch (const std::exception& e)
    {
      throw DatabaseError(e.what());
    }
}

pqxx::result
transaction::exec(const std::string& query)
{
//   std::cerr << "transaction::exec" << std::endl;
  try
    {
      if (m_state == STATE_EXECUTING)
	return m_transaction->exec(query);
//       else if (m_state == STATE_ABORTING || m_state == STATE_COMMITTING)
// 	throw Abort
      else
	throw DatabaseError("The query could not be executed: " + describe_status(m_state));
    }
  catch (const std::exception& e)
    {
      throw DatabaseError(e.what());
    }
}

pqxx::result::size_type
transaction::exec_noresult(const std::string& query)
{
//   std::cerr << "transaction::exec_noresult" << std::endl;
  try
    {
      if (m_state == STATE_EXECUTING)
	{
	  pqxx::result R = exec(query);
	  return R.affected_rows();
	}
//       else if (m_state == STATE_ABORTING || m_state == STATE_COMMITTING)
// 	throw Abort
      else
	throw DatabaseError("The query could not be executed: " + describe_status(m_state));
    }
  catch (const std::exception& e)
    {
      throw DatabaseError(e.what());
    }
}

pqxx::result::size_type
transaction::perform(const std::string& query,
		     pqxx::result::size_type min_rows,
		     pqxx::result::size_type max_rows)
{
//   std::cerr << "transaction::perform" << std::endl;
  try
    {
      if (m_state == STATE_EXECUTING)
	{
	  begin("transaction::perform()");

	  pqxx::result::size_type rows;
	  rows = exec_noresult(query);

	  // Commit if the number of altered rows is greater or equal to
	  // min_rows, or if min_rows is 0.
	  if ((rows >= min_rows || min_rows == 0) &&
	      (rows <= max_rows || max_rows == 0))
	    commit();
	  else
	    abort();

	  return rows;
	}
      else
//       else if (m_state == STATE_ABORTING || m_state == STATE_COMMITTING)
// 	throw Abort
	throw DatabaseError("The query could not be executed: " + describe_status(m_state));
    }
  catch (const std::exception& e)
    {
      _end();
      throw DatabaseError(e.what());
    }

  // This should never be reached.
  return 0;
}

void
transaction::commit()
{
  commit(true);
}

void
transaction::abort()
{
  abort(true);
}

void
transaction::_begin(const std::string& name)
{
//   std::cerr << "transaction::_begin" << std::endl;
  assert (m_transaction == NULL);

  m_transaction = new pqxx::transaction<> (m_connection, name);
  m_state = STATE_EXECUTING;
}

void
transaction::_end()
{
//   std::cerr << "transaction::_end" << std::endl;
  if (m_transaction)
    {
      delete m_transaction;
      m_transaction = 0;
      m_state = STATE_NONE;
      m_recursion_level = 0;
    }
}

void transaction::commit(bool refresh)
{
//   std::cerr << "transaction::commit" << std::endl;
  try
    {
      if (m_recursion_level == 1)
	{
	  _commit(refresh);
	}
      else // end() is implict above
	end();
    }
  catch (const std::exception& e)
    {
      _end();
      throw DatabaseError(e.what());
    }
}

void
transaction::_commit(bool refresh)
{
  m_state = STATE_COMMITTING;
  m_transaction->commit();
  m_state = STATE_COMMITTED;
  end();

  // Start a new transaction for object refresh phase.
  if (refresh == true)
    {
      begin("pqxxobject::commit[object refresh]");
      m_signal_commit.emit(*this);
      m_signal_commit.clear();
      m_signal_abort.clear();
      commit(false);
    }
}

void
transaction::abort(bool refresh)
{
//   std::cerr << "transaction::abort" << std::endl;
  try
    {
      m_state = STATE_ABORTING;
      if (m_recursion_level == 1)
	{
	  _abort(refresh);
	}
      else // end() is implict above
	end();
    }
  catch (const std::exception& e)
    {
      _end();
      throw DatabaseError(e.what());
    }
}
void
transaction::_abort(bool refresh)
{
  m_state = STATE_ABORTING;
  m_transaction->abort();
  m_state = STATE_ABORTED;
  end();

  // Start a new transaction for object refresh phase.
  if (refresh == true)
    {
      begin("pqxxobject::abort[object refresh]");
      m_signal_commit.emit(*this);
      m_signal_commit.clear();
      m_signal_abort.clear();
      m_transaction->commit();
      commit(false);
    }
}

