// tutorial places driver program                                -*- C++ -*-
// $Id: places-main.cc,v 1.6 2004/01/28 21:21:10 roger Exp $
//
// Copyright (C) 2003  Roger Leigh <rleigh@debian.org>
//
//
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//  * Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above
//    copyright notice, this list of conditions and the following
//    disclaimer in the documentation and/or other materials provided
//    with the distribution.
//  * Neither the name of the author, nor the names of other
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
//  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
//  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
//  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
//  SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////

#include <iomanip>
#include <iostream>

#include "places.h"

using namespace pqxx;

void print_header()
{
  std::cout << " id  |        name        | gridref \n"
	    << "-----+--------------------+---------\n";
}

void print_place(const Place& place)
{
  std::cout << std::setw(4) << std::setfill(' ') << std::right << place.get_id() << " | "
	    << std::setw(18) << std::setfill('.') << std::left << place.get_name()
	    << " | " << place.get_gridref() << "\n";
}

void print_list(const PlaceTable::row_list_ptr& list)
{
  print_header();
  for (PlaceTable::row_list::const_iterator cur = list->begin();
       cur != list->end();
       ++cur)
    print_place(*cur);
  std::cout << "(" << list->size() << " rows)\n";
}

int main()
{
  try
    {
      // Create a new database connection.
      std::cout << "Creating a database connection..." << std::flush;
      connection C("dbname=libpqxx_object_tutorial");
      std::cout <<"done.\n";

      // Create a PlaceTable object (represents the places table).
      std::cout << "Creating a PlaceTable..." << std::flush;
      pqxx::transaction<> *T = 0;
      pqxxobject::transaction tran(C, T);
      PlaceTable myplaces(tran);
      std::cout <<"done.\n\n";

      // Clean up database, so this program can be idempotent.
      PlaceTable::row_ptr delplace = myplaces.find_name("Inveraray");
      if (delplace.get() != NULL)
	myplaces.erase(*delplace);

      // Print a list of all the places in the table.
      std::cout << "A list of all places in the places database table.\n";
      PlaceTable::row_list_ptr list = myplaces.get_list();

      print_list(list);

      // Print a list of all the places in the table, sorted by grid reference
      std::cout << "\nA list of all places in the places database table, sorted by grid reference\n";
      list = myplaces.get_list(PlaceTable::ORDER_GRIDREF);

      print_list(list);

      // Find a place by ID.
      std::cout << "\nFinding a place by its ID number.\n";
      PlaceTable::row_ptr place = myplaces.find(4);
      if (place.get() != NULL)
	{
	  print_header();
	  print_place(*place);
	}
      else
	std::cout << "No place found\n";

      // Find a place by name.
      std::cout << "\nFinding a place by its name.\n";
      place = myplaces.find_name("Kirkwall");
      if (place.get() != NULL)
	{
	  print_header();
	  print_place(*place);
	}
      else
	std::cout << "No place found\n";

      // Find a place by grid reference.
      std::cout << "\nFinding a place by its grid reference.\n";
      list = myplaces.find_gridref("SM9537");

      print_list(list);

      // Insert a new place.
      std::cout << "\nInserting a new place: Inveraray, grid reference NB9999\n";
      Place newplace("Inveraray", "NB9999");
      std::cout << "Row state before insertion: id = " << newplace.get_id()
		<< ", state = " << newplace.get_state() << "\n";
      myplaces.insert(newplace);
      std::cout << "Row state after insertion: id = " << newplace.get_id()
		<< ", state = " << newplace.get_state() << "\n";

      // Check the insertion worked.
      std::cout << "\nCheck the insertion worked.\n";
      place = myplaces.find_name("Inveraray");
      if (place.get() != NULL)
	{
	  print_header();
	  print_place(*place);
	}
      else
	std::cout << "No place found\n";

      // Update the place we found previously.
      std::cout << "\nUpdating the row: the grid reference should be NN0908\n";
      if (place.get() != NULL)
	{
	  place->set_gridref("NN0908");
	  myplaces.update(*place);
	}
      else
	std::cout << "No place to update\n";

      // Check the update worked.
      std::cout << "\nCheck the update worked.\n";
      place = myplaces.find_name("Inveraray");
      if (place.get() != NULL)
	{
	  print_header();
	  print_place(*place);
	  std::cout << "Row state: id = " << place->get_id()
		    << ", state = " << place->get_state() << "\n";
	}
      else
	std::cout << "No place found\n";

      // Now try to delete a place.
      std::cout << "\nNow delete the row.\n";
      if (place.get() != NULL)
	{
	  myplaces.erase(*place);
	  std::cout << "Row state after deletion: id = " << place->get_id()
		    << ", state = " << place->get_state() << "\n";
	}
      else
	std::cout << "No place to delete\n";

      // Check the deletion worked.
      std::cout << "\nCheck the deletion worked (no row should be found).\n";
      place = myplaces.find_name("Inveraray");
      if (place.get() != NULL)
	{
	  print_header();
	  print_place(*place);
	}
      else
	std::cout << "No place found\n";
    }
  catch (const pqxxobject::DatabaseError& e)
    {
      std::cerr << "\nCaught a DatabaseError: " << e.what() << std::endl;
      return 1;
    }
  catch (const std::exception& e)
    {
      std::cerr << "\nCaught an exception: " << e.what() << std::endl;
      return 1;
    }
  catch (...)
    {
      std::cerr << "\nCaught an unknown exception; terminating now." << std::endl;
      return 1;
    }
  return 0;
}
