// tutorial places class                                         -*- C++ -*-
// $Id: places.h,v 1.10 2004/01/28 21:21:10 roger Exp $
//
// Copyright (C) 2003  Roger Leigh <rleigh@debian.org>
//
//
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//  * Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above
//    copyright notice, this list of conditions and the following
//    disclaimer in the documentation and/or other materials provided
//    with the distribution.
//  * Neither the name of the author, nor the names of other
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
//  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
//  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
//  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
//  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
//  SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////

#ifndef TUTORIAL_PLACES_H
#define TUTORIAL_PLACES_H

#include <string>
#include <vector>

#include <pqxx-object/field.h>
#include <pqxx-object/row.h>
#include <pqxx-object/table.h>

/**
 * Place details.
 * This class stores the details of a single place.  It represents a
 * single row in the places database table.  The place id number, name
 * and OS grid reference are known.  Each may be accessed and changed
 * using the class methods, with the exception of setting the id
 * number (which is set by the backend database).
 */
class Place : public pqxxobject::row<Place>
{
public:
  /// The constructor.
  Place();

  /**
   * The constructor.
   * @param name the place name.
   * @param gridref the place grid reference.
   */
  Place(const std::string& name,
	const std::string& gridref);

  /// The destructor.
  virtual ~Place();

  /**
   * Get the place id number.
   * @returns the place id.
   */
  int get_id() const;

  /**
   * Get the place name.
   * @returns a constant reference to the name.
   */
  const std::string& get_name() const;

  /**
   * Set the place name.
   * @param name the place name.
   */
  void set_name(const std::string& name);

  /**
   * Get the place grid reference.
   * @returns a constant reference to the grid reference.
   */
  const std::string& get_gridref() const;

  /**
   * Set the place grid reference.
   * @param gridref the grid reference
   */
  void set_gridref(const std::string& gridref);

protected:
  /**
   * Insert a row into a table.
   */
  virtual void insert_impl(pqxxobject::transaction& tran);

  /**
   * Update a row from a table.
   */
  virtual void update_impl(pqxxobject::transaction& tran);

  /**
   * Remove a row from a table.
   */
  virtual void erase_impl(pqxxobject::transaction& tran);

  /**
   * Refresh a row from a table.
   */
  virtual void refresh_impl(pqxxobject::transaction& tran);

  virtual void convert_impl(pqxx::result::const_iterator row,
			    pqxxobject::transaction& tran);

private:
  /// ID number.
  pqxxobject::field<int> m_id;
  /// Name.
  pqxxobject::field<std::string> m_name;
  /// Grid Reference.
  pqxxobject::field<std::string> m_gridref;

  friend class pqxxobject::row<Place>;

}; // class Place

/**
 * Place database interaction.
 * This class represents the places database table, and may be used to
 * obtain information about places held in the place table.  It may be
 * used to add new places, update information about existing places,
 * and remove places.
 */
class PlaceTable : public pqxxobject::table<Place>
{
public:
  /// Underlying table type.
  typedef pqxxobject::table<Place> table_base;

  /**
   * The constructor.
   * @param tran a PostgreSQL transaction object.
   */
  PlaceTable(pqxxobject::transaction& tran);

  /// The destructor.
  virtual ~PlaceTable();

  /// Sort order
  enum sort_order
    {
      /// Order by ID number.
      ORDER_ID,
      /// Order by name.
      ORDER_NAME,
      /// Order by grid reference.
      ORDER_GRIDREF
    };

  /**
   * Get a list of all places.
   * @param order the order to sort the list in.
   * @returns a pointer to a vector of places.
   */
  row_list_ptr get_list(sort_order order = ORDER_NAME);

  /**
   * Find a single place by its ID number.
   * @param place_id the place ID.
   * @returns a pointer to the place, or NULL if no place was found.
   */
  row_ptr find(int place_id);

  /**
   * Find a place by its name.
   * @param name the place name.
   * @returns a pointer to the place, or NULL if no place was found.
   */
  row_ptr find_name(const std::string& name);

  /**
   * Find places by their grid reference.
   * Because there is no garuantee that the grid reference is
   * unique, this function may return zero, one or many places.
   * @param gridref the place grid reference.
   * @returns a pointer to a vector of places.
   */
  row_list_ptr find_gridref(const std::string& gridref);

}; // class PlaceTable

#endif // TUTORIAL_PLACES_H
